#include <unistd.h>
#include <time.h>
#include <X11/Xlib.h>
#include <sys/time.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "plugin.h"
#include <background.h>
#include "arrow.xpm"

#define OFFSET 12

typedef struct _tagRECT
{
  int left;
  int top;
  int right;
  int bottom;
} RECT,*LPRECT;

typedef struct _tagBGMODULE
{
  MODULE base;

  /* member info */
  Display *pDisplay;
  Window root;
  Window panelWindow;
  int screen;
  unsigned long backGround;
  int width;
  int height;
  RECT arrowRect;
  GC gc;
  PIXMAPFUNCS pixmapFuncs;
  PIXMAP icon;
  unsigned int timeout;
  char socket[256];
  char currentImage[256];
} BGMODULE,*LPBGMODULE;

static void LoadSettings(LPBGMODULE pBg,LPCONFIGITEM pOptions)
{
  LPCONFIGITEM pItem;

  /* load defaults */
  strcpy(pBg->socket,"/tmp/background");
  pBg->timeout = 1000;
  /* parse options */
  if (pOptions && pOptions->pSublist)
  {
    for (pItem=(LPCONFIGITEM)pOptions->pSublist->list.pHead;pItem;pItem = (LPCONFIGITEM)pItem->node.pNext)
    {
      if (strcmp(pItem->name,"socket") == 0)
      {
	strcpy(pBg->socket,pItem->value);
      }
      else if (strcmp(pItem->name,"timeout") == 0)
      {
	pBg->timeout = atoi(pItem->value);
      }
    }
  }
}

static void GetCurrent(LPBGMODULE pBg)
{
  LPBACKGROUND pback = backgroundCreate(pBg->socket,pBg->timeout);
  if (pback)
  {
    backgroundGetCurrent(pback,pBg->currentImage);
    backgroundDestroy(pback);
  }
}

static void NextImage(LPBGMODULE pBg)
{
  LPBACKGROUND pback = backgroundCreate(pBg->socket,pBg->timeout);
  if (pback)
  {
    backgroundNext(pback);
    backgroundDestroy(pback);
  }
}

static void DrawBackground(LPBGMODULE pBg,int drawBackground)
{
  if (drawBackground)
  {
    XClearArea(pBg->pDisplay,pBg->base.panels[0].wnd,0,0,0,0,False);
  }

  pBg->pixmapFuncs.RenderPixmap(pBg->pDisplay,pBg->base.panels[0].wnd,pBg->gc,
				pBg->arrowRect.left,pBg->arrowRect.top,
				&pBg->icon);
}

static const char *BG_GetTooltip(LPBGMODULE pBg)
{
  *pBg->currentImage = 0;	/* in case of failure */
  GetCurrent(pBg);
  return pBg->currentImage;
}

static void BG_EventHandler(LPBGMODULE pBg,XEvent *pEvent)
{
  switch (pEvent->type)
  {
  case ButtonPress:
    if (pEvent->xbutton.button == 1)
    {
      NextImage(pBg);
    }
    break;
  case Expose:
    if (pEvent->xexpose.count == 0)
    {
      DrawBackground(pBg,0);
    }
  }
}

static void BG_Destroy(LPBGMODULE pBg)
{
#ifdef DEBUG
  printf("background.so, Destroy()\n");
#endif
  if (pBg->gc)
  {
    XFreeGC(pBg->pDisplay,pBg->gc);
  }
  
  pBg->pixmapFuncs.FreePixmap(&pBg->icon);

  free(pBg);
}

static Window CreateWindow(LPBGMODULE pBg,LPLIBRARYINIT pInit)
{
  XSetWindowAttributes att;
  XGCValues gcv;
  Window ret;
  
  memset(&att,0,sizeof(att));
  
#ifdef DEBUG
  printf("Creating background window with a height of %i\n",pInit->height);
#endif
#define BASE_LEN "23:55 AM"
  pBg->width = 20;
  pBg->height = pInit->height;

  att.background_pixel = pInit->backgroundPixel;
  att.event_mask = pInit->baseInputMask | ButtonPressMask | ExposureMask;

  ret = XCreateWindow(pInit->pDisplay,pInit->panelWindow,0,0,pBg->width,pInit->height,0,CopyFromParent,InputOutput,CopyFromParent,CWEventMask | CWBackPixel,&att);
  if (ret)
  {
    pBg->arrowRect.left = (pBg->width - 12)/2;
    pBg->arrowRect.top  = (pBg->height - 10)/2;
    pBg->arrowRect.right = pBg->arrowRect.left + 12;
    pBg->arrowRect.bottom = pBg->arrowRect.top + 10;

    /* load the xpm */
    pBg->pixmapFuncs.LoadPixmap(arrow_xpm,&pBg->icon);
    /* setup gc */
    gcv.graphics_exposures = False;
    gcv.clip_mask = pBg->icon.mask;
    gcv.clip_x_origin = pBg->arrowRect.left;
    gcv.clip_y_origin = pBg->arrowRect.top;
    pBg->gc = XCreateGC(pInit->pDisplay,ret,GCGraphicsExposures | GCClipMask | GCClipXOrigin | GCClipYOrigin,&gcv);
  }
  return ret;
}

int Init(LPLIBRARYINIT pInit)
{
  LPBGMODULE pBg = (LPBGMODULE)malloc(sizeof(BGMODULE));
  if (pBg)
  {
    memset(pBg,0,sizeof(BGMODULE));

    pBg->base.Destroy = (MODULE_DESTROY)BG_Destroy;
    pBg->base.Timeout = NULL;
    pBg->base.EventHandler = (MODULE_EVENTHANDLER)BG_EventHandler;
    pBg->base.GetTooltip = (MODULE_GETTOOLTIP)BG_GetTooltip;
    pBg->base.numPanels = 1;

    memcpy(&pBg->pixmapFuncs,&pInit->pixmapFuncs,sizeof(PIXMAPFUNCS));

    pBg->pDisplay = pInit->pDisplay;
    pBg->root     = pInit->root;
    pBg->screen   = pInit->screen;
    pBg->panelWindow = pInit->panelWindow;
    pBg->backGround = pInit->backgroundPixel;

    LoadSettings(pBg,pInit->pOptions);

    pBg->base.panels[0].name = "Background";
    if (!(pBg->base.panels[0].wnd  = CreateWindow(pBg,pInit)))
    {
      free(pBg);
      return 0;
    }
    /* set return value */
    pInit->pModule = (LPMODULE)pBg;
  }
  return (pBg != 0);
}
